require 'faraday'
require 'json'
require 'eventmachine'
require 'em-http'

require 'analytics-psw/reports'
require 'analytics-psw/services'
require 'analytics-psw/event_types'
require 'analytics-psw/events'
require 'analytics-psw/dimension_properties'
require 'analytics-psw/dimensions'
require 'analytics-psw/customers'
require 'analytics-psw/utilities'
require 'analytics-psw/recent_events'
require 'analytics-psw/api'
require 'analytics-psw/railtie'
require 'analytics-psw/rack/request_logging'
require 'analytics-psw/rack/request_logger'

module AnalyticsPSW
  class Analytics

    include AnalyticsPSW::Customers,
            AnalyticsPSW::Reports,
            AnalyticsPSW::Services,
            AnalyticsPSW::EventTypes,
            AnalyticsPSW::Events,
            AnalyticsPSW::Dimensions,
            AnalyticsPSW::DimensionProperties,
            AnalyticsPSW::Utilities,
            AnalyticsPSW::RecentEvents

    attr_accessor :service_location, :event_types, :dimensions

    def self.get(opts = {})
      klass = opts['mock'] ? MockAnalytics : Analytics
      klass.new(opts)
    end

    def initialize(opts = {})
      set_vars_from_opts(opts)
      @conn = faraday_connection
      update_token if @client_id && @client_secret
      @service_location = "#{@server_url}/services/#{@service_name}"
      resp = http_request(:get, @service_location)
      raise 'The service name provided does not appear to be valid' unless resp['status'] == 200
      find_or_create_event_types
      find_or_create_dimensions(@dimensions) if @dimensions
    end

    private

    def set_vars_from_opts(opts)
      @server_url = opts['server_url'] ? opts['server_url'].chomp('/') : nil
      @service_name = opts['service']
      raise 'Please ensure you have provided the appropriate server url and service name' unless @server_url && @service_name && @service_name != ''
      @proxy = opts['proxy']
      @proxy = '' if @server_url.start_with?('http://127.0.0.1') or @server_url.start_with?('http://localhost')
      @event_types = opts['event_types'] || []
      @dimensions = opts['dimensions'] || []
      @client_id = opts['client_id']
      @client_secret = opts['client_secret']
    end
  end

  class MockAnalytics
    attr_accessor :event_types, :dimensions

    def initialize(opts = {})
      debug("initializing mock Analytics")
      @event_types = opts['event_types'] || []
      @dimensions = opts['dimensions'] ? (opts['dimensions'].map { |hsh| hsh.keys }.flatten) : []
    end

    def record_event(event_type, metadata = {}, asynchronous = false)
      debug("record event: #{event_type}, #{metadata.inspect}")
    end

    def debug(msg)
      return if !defined?(Rails) || Rails.logger.blank?

      Rails.logger.info("Analytics: " + msg)
    end
  end
end
