module AnalyticsPSW
  class Api
    def initialize(server_url, client_id, client_secret, proxy = nil)
      @server_url = server_url
      @client_id = client_id
      @client_secret = client_secret
      @proxy = proxy
    end

    def connection
      opts = {}
      opts[:proxy] = { uri: @proxy } if @proxy
      Faraday.new(@server_url, opts)
    end

    def access_token_hash
      response_hash = {}
      response = connection.post do |request|
        request.url "#{@server_url}/oauth/token?client_id=#{@client_id}&client_secret=#{@client_secret}"
        request.headers['Content-Type'] = 'application/json'
      end
      parsed_response = self.class.parse_json_response(response)
      if parsed_response['body'].present?
        response_hash[:token] = parsed_response['body']['access_token']
        response_hash[:expires] = Time.now + parsed_response['body']['expires_in']
      end
      response_hash
    end

    def self.parse_json_response(json_response)
      parsed_response = {}
      parsed_response['status'] = json_response.status if json_response.status.present?

      if [200, 201, 202].include?(json_response.status)
        parsed_response['body'] = JSON.parse(json_response.body) if json_response.body.present?
        parsed_response['location'] = json_response.headers['location'] if json_response.headers.present? && json_response.headers['location']
      else
        parsed_response['body'] = {}
        parsed_response['error_message'] = json_response.body
      end

      parsed_response
    end
  end
end
