module AnalyticsPSW
  module DimensionProperties
    def list_dimension_properties(dimension_name)
      http_request(:get, "#{@service_location}/dimensions/#{dimension_name}/properties")
    end

    def show_dimension_properties(dimension_name, dimension_value)
      http_request(:get, "#{@service_location}/dimensions/#{dimension_name}/properties/#{dimension_value}")
    end

    def create_dimension_properties(dimension_name, dimension_value, properties = {})
      metadata = create_dimension_properties_metadata(dimension_value, properties)
      body = create_json_body('dimension_property', metadata)
      http_request(:post, "#{@service_location}/dimensions/#{dimension_name}/properties/#{dimension_value}", nil, body)
    end

    def update_dimension_properties(dimension_name, dimension_value, properties = {})
      metadata = create_dimension_properties_metadata(dimension_value, properties)
      body = create_json_body('dimension_property', metadata)
      http_request(:put, "#{@service_location}/dimensions/#{dimension_name}/properties/#{dimension_value}", nil, body)
    end

    def create_or_update_dimension_properties(dimension_name, dimension_value, properties = {})
      dimension_property_exists?(dimension_name, dimension_value) ? update_dimension_properties(dimension_name, dimension_value, properties) : create_dimension_properties(dimension_name, dimension_value, properties)
    end

    private

    def create_dimension_properties_metadata(dimension_value, properties)
      dimension_properties_body = {'value' => dimension_value, 'properties' => {}}
      properties.each do |key, value|
        dimension_properties_body['properties'][key] = value unless value == ''
      end
      dimension_properties_body
    end

    def dimension_property_exists? (dimension_name, dimension_value)
      dim_properties = list_dimension_properties(dimension_name)['body']
      dim_properties[dimension_name].present? && dim_properties[dimension_name] == dimension_value
    end
  end
end
