module AnalyticsPSW
  module Events

    def record_event(event_type, metadata = {}, asynchronous = false)
      return invalid_event_type_response unless event_type_exists? event_type
      return invalid_metadata_response unless dimensions_exist? metadata

      _record_event = -> do
        metadata[:event_type] = event_type.to_s
        body = create_json_body('event', metadata)
        http_request(:post, "#{@service_location}/events", nil, body)
      end

      asynchronous ? Thread.new { _record_event.call } : _record_event.call
    end

  private

    def event_type_exists?(current_event_type)
      @event_types.include? current_event_type.to_s
    end

    def dimensions_exist?(metadata)
      metadata.keys.each do |key|
        return false unless @dimensions.include? key.to_s
      end
    end

    def invalid_event_type_response
      {'status' => 422, 'body' => {}, 'error_message' => 'Invalid event type for service.'}
    end

    def invalid_metadata_response
      {'status' => 422, 'body' => {}, 'error_message' => 'Invalid dimensions for service.'}
    end

  end
end

