require 'benchmark'

module AnalyticsPSW::Rack
  class RequestLogger
    include AnalyticsPSW::Rack::RequestLogging

    def initialize(app)
      @app = app
    end

    def call(env)
      @status, headers, body = call_with_benchmark do
        @app.call(@env = env)
      end

      log_request

      [ @status, headers, body ]
    end

    private

    def call_with_benchmark
      value = nil
      @request_time = Benchmark.measure do
        value = yield
      end.real
      value
    end

    def params
      @env['action_dispatch.request.parameters']
    end

    def headers
      @env
    end

    def analytics_service
      @env['request_logging.analytics_service']
    end

    def log_request_response_params
      {
       'response_code' => @status,
       'request_time' => @request_time,
      }
    end

    def log_request_application_params
      @env['request_logging.application_params'] || {}
    end

    def request
      Rack::Request.new(@env)
    end

    def analytics_load
      if @env['request_logging.load']
        @env['request_logging.load'].call
      end
    end
  end
end
