module AnalyticsPSW::Rack
  module RequestLogging
    protected

    def log_request
      return unless analytics_load
      return unless analytics_service
      return unless analytics_event_types.include?(analytics_event_type)
      analytics_service.record_event(analytics_event_type, log_request_params, true)
    end

    def analytics_service
      nil
    end

    def analytics_event_type
      'api_call'
    end

    def analytics_event_types
      analytics_service.event_types
    end

    def analytics_dimensions
      analytics_service.dimensions
    end

    def log_request_headers
      %w(HTTP_USER_AGENT REQUEST_PATH PATH_INFO SERVER_NAME REMOTE_HOST REQUEST_METHOD rack.url_scheme)
    end

    def log_request_header_params
      log_request_headers.inject({}) do |hsh, k|
        hsh[k.downcase.tr('.', '_')] = headers[k]
        hsh
      end
    end

    def log_request_route_params
      { canonical_request_path: canonical_request_path }
    end

    def canonical_request_path
      return nil unless defined?(Rails)
      return nil unless Rails.application.routes.respond_to?(:router)

      Rails.application.routes.router.recognize(request) do |route, _|
        return route.path.spec.to_s
      end
    end

    def log_request_response_code
      response.code
    end

    def log_request_server_params
      { host_name: Socket.gethostname }
    end

    def log_request_available_params
      params
        .merge(log_request_server_params)
        .merge(log_request_header_params)
        .merge(log_request_route_params)
        .merge(log_request_response_params)
        .merge(log_request_application_params)
    end

    def log_request_params
      log_request_available_params.select do |k|
        analytics_dimensions.include?(k.to_s)
      end
    end
  end
end
