require 'analytics_report'
require 'analytics_report_field'

module AnalyticsPSW
  module Reports

    def query_analytics(query_parameters)
      report_response = http_request(:get, "#{@service_location}/reports/query.json", query_parameters)
      create_analytics_report(report_response)
    end

    def multi_query_analytics(query_parameters)
      raw_reports = multi_get_json("#{@service_location}/reports/query.json", query_parameters)
      raw_reports.map do |raw_report|
        create_analytics_report(raw_report)
      end

    end

    def create_report(name, query_parameters)
      query_parameters[:name] = name.to_s
      body = create_json_body('report', query_parameters)
      http_request(:post, "#{@service_location}/reports", nil, body)
    end

    def list_reports
      http_request(:get, "#{@service_location}/reports")
    end

    def report_criteria(report_name)
      http_request(:get, "#{@service_location}/reports/#{report_name}/criteria")
    end

    def run_report(report_name, where_criteria = {})
      report_response = http_request(:get, "#{@service_location}/reports/#{report_name}/results", where_criteria)
      create_analytics_report(report_response)
    end

    def delete_report(report_name)
      http_request(:delete, "#{@service_location}/reports/#{report_name}")
    end

private

    def create_analytics_report(raw_report)
      analytics_params = {status: raw_report['status']}
      if analytics_params[:status] == 200
        analytics_params[:rows] = convert_body_to_rows(raw_report['body'])
      else
        analytics_params[:rows] = [{}]
        analytics_params[:error_message] = raw_report['error_message']
      end
      AnalyticsReport.new(analytics_params)
    end

    def convert_body_to_rows(body)
      body.map {|row|
        fields = Hash[row.map {|field, value|
          field_params = {name: field, value: value, is_aggregate: value.class == Hash}
          [field.to_sym, AnalyticsReportField.new(field_params)]
        }]
        fields
      }
    end
  end
end
