module AnalyticsPSW
  module Utilities
    private

    def update_token
      access_token_hash = Api.new(@server_url, @client_id, @client_secret, @proxy).access_token_hash
      @token = access_token_hash[:token]
      @expires = access_token_hash[:expires]
    end

    def faraday_connection
      Api.new(@server_url, @client_id, @client_secret, @proxy).connection
    end

    def http_request(http_method, url, params = nil, body = nil, retries = 2)
      while retries > 0
        json_block  = ->(req) do
          req.url url
          req.headers['Content-Type'] = 'application/json'
          req.headers['Authorization']= "Bearer #{@token}" if @token
          req.params = params if params
          req.body = body if body
        end
        parsed_response = parse_json_response(@conn.send(http_method, &json_block))
        break if [200, 201, 202].include?(parsed_response['status'])
        update_token if parsed_response['status'] == 401
        retries -= 1
      end
      parsed_response
    end
    
    def multi_get_json(service_location, query_parameters)
      update_token if need_token_update?
      results = []
      EventMachine.run do
        multi = EventMachine::MultiRequest.new
        connection_opts = {}
        connection_opts = {proxy: {host: @proxy.sub(/^https?\:\/\//, ''), port: 80}} if @proxy.present?
        query_parameters.each_with_index do |param, idx|
          http = EventMachine::HttpRequest.new(service_location, connection_opts)
          req = http.get query: param, head: {'Authorization' => "Bearer #{@token}"}
          multi.add idx, req
        end
        multi.callback do
          response = multi.responses[:callback]
          error =  multi.responses[:errback]
          EventMachine.stop
          (0..query_parameters.size-1).each do |idx|
            results += [multi_parse_json_response(response[idx])] if response[idx].present?
            results += [multi_parse_json_response(error[idx])] if error[idx].present?
          end
        end
      end
      results
    end

    def create_json_body(type, metadata = {})
      {type => metadata.reject { |k,v| v == '' }}.to_json
    end

    def parse_json_response(json_response)
      Api.parse_json_response(json_response)
    end

    def multi_parse_json_response(response)
      parsed_response = {}
      parsed_response['status'] = response.response_header.present? ? response.response_header.status : 500

      if [200, 201, 202].include?(parsed_response['status'])
        parsed_response['body'] = JSON.parse response.response if response.response.present?
        parsed_response['location'] = response.headers.location if response.headers.present? && response.headers.location.present?
      else
        parsed_response['body'] = {}
        parsed_response['error_message'] = response.error if response.error.present?
      end
      parsed_response
    end
    
    def need_token_update?
      (@client_id and @client_secret and @expires.nil?) or (@expires and Time.now > @expires)
    end
  end
end
