namespace :analytics do
  def dump_report(event_type, report)
    if report.rows.count == 0
      puts "Report has no rows"
      return
    end

    header_columns = report.rows.first.keys.map { |k| k == event_type.to_sym ? 'count' : k }
    rows = report.rows.map do |report_row|
      row = report_row.map do |key, analytics_report_field|
        if analytics_report_field.value.is_a?(Hash)
          analytics_report_field.value['sum']
        else
          analytics_report_field.value.inspect
        end
      end
    end

    puts header_columns.join("\t")
    rows.each do |row|
      puts row.join("\t")
    end
  end

  task :group_by do |t, args|
    analytics, event_type, key = *args
    begin_date ||= ENV['begin_date'] || 1.week.ago.to_date
    end_date ||= ENV['end_date'] || Date.today
    puts "Group by #{key}:"
    opts = {
      sum: event_type,
    }
    if %w(hour day month year).include?(key)
      opts[:iterate_by] = key.upcase
    else
      opts[:fields] = key
      opts[:group_by] = key
      opts[:order_by] = "#{event_type} descending"
    end
    opts[:where] = [
                    "event_time gte #{begin_date}",
                    "event_time lte #{end_date}T23:59:59+00:00",
                   ]
    dump_report(event_type, analytics.query_analytics(opts))
    puts ""
  end
end

def analytics_define_group_by_tasks(analytics, event_type, dimensions)
  namespace :analytics do
    (dimensions.map { |hsh| hsh.map { |key, _type| key } } + %w(hour day month year)).flatten.each do |key|
      desc "Display analytics report grouped by #{key}"
      task "group_by_#{key}" do
        task("analytics:group_by").execute([analytics, event_type, key])
      end
    end

    desc "Display all analytics reports"
    task all: Rake.application.tasks.select { |t| (t.scope.is_a?(Array) ? t.scope == [ "analytics" ] : t.scope.path == "analytics") && t.name != 'analytics:group_by' }.map(&:name)
  end
end
